package en.MaxedOut.UltraRespawn.Utils;

import org.bukkit.World;
import org.bukkit.util.BoundingBox;
import org.bukkit.util.Vector;
import java.util.ArrayList;
import org.bukkit.event.player.PlayerQuitEvent;
import org.bukkit.event.EventHandler;
import org.bukkit.inventory.ItemStack;
import org.bukkit.event.entity.ProjectileLaunchEvent;
import java.util.Iterator;
import java.util.List;
import org.bukkit.projectiles.ProjectileSource;
import org.bukkit.Material;
import org.bukkit.block.Block;
import org.bukkit.entity.Trident;
import org.bukkit.entity.Egg;
import org.bukkit.entity.EnderPearl;
import org.bukkit.entity.Snowball;
import org.bukkit.entity.Arrow;
import org.bukkit.entity.Entity;
import org.bukkit.entity.Projectile;
import org.bukkit.event.entity.EntityDamageByEntityEvent;
import org.bukkit.event.entity.EntityDamageEvent;
import org.bukkit.entity.Player;
import java.util.HashMap;
import org.bukkit.event.Listener;

public class DeathMessageHandler implements Listener
{
    private HashMap<Player, String> bowNames;
    
    public DeathMessageHandler() {
        this.bowNames = new HashMap<Player, String>();
    }
    
    public String getDeathMessage(final Player p, final EntityDamageEvent e) {
        if (e instanceof EntityDamageByEntityEvent) {
            final EntityDamageByEntityEvent event = (EntityDamageByEntityEvent)e;
            final Entity damager = event.getDamager();
            if (!(damager instanceof Projectile)) {
                return p.getName() + " was killed by " + damager.getName();
            }
            final Projectile projectile = (Projectile)damager;
            String shooterName = projectile.getType().getName();
            final ProjectileSource shooter = projectile.getShooter();
            if (shooter instanceof Entity) {
                shooterName = ((Entity)shooter).getName();
                if (projectile instanceof Arrow && shooter instanceof Player) {
                    final Player shooterP = (Player)shooter;
                    if (this.bowNames.containsKey(shooterP)) {
                        shooterName = shooterName + " using " + this.bowNames.get(shooterP);
                        this.bowNames.remove(shooterP);
                    }
                }
            }
            if (projectile instanceof Snowball || projectile instanceof EnderPearl || projectile instanceof Egg) {
                return p.getName() + " was pummeled by " + shooterName;
            }
            if (projectile instanceof Arrow) {
                return p.getName() + " was shot by " + shooterName;
            }
            if (projectile instanceof Trident) {
                return p.getName() + " was impaled by " + shooterName;
            }
        }
        else {
            final EntityDamageEvent.DamageCause cause = e.getCause();
            if (cause == EntityDamageEvent.DamageCause.BLOCK_EXPLOSION) {
                return p.getName() + " blew up";
            }
            if (cause == EntityDamageEvent.DamageCause.CONTACT) {
                final List<Block> b = this.getBlocksInHitbox(p);
                for (final Block block : b) {
                    if (block.getType() == Material.CACTUS) {
                        return p.getName() + " was pricked to death";
                    }
                    if (block.getType() == Material.SWEET_BERRY_BUSH) {
                        return p.getName() + " was poked to death by a sweet berry bush";
                    }
                    if (block.getType() == Material.POINTED_DRIPSTONE) {
                        return p.getName() + " was pricked to death";
                    }
                }
                return p.getName() + " died";
            }
            if (cause == EntityDamageEvent.DamageCause.CRAMMING) {
                return p.getName() + " was squished too much";
            }
            if (cause == EntityDamageEvent.DamageCause.DRAGON_BREATH) {
                return p.getName() + " was roasted in dragon breath";
            }
            if (cause == EntityDamageEvent.DamageCause.DROWNING) {
                return p.getName() + " drowned";
            }
            if (cause == EntityDamageEvent.DamageCause.FALL) {
                return p.getName() + " hit the ground too hard";
            }
            if (cause == EntityDamageEvent.DamageCause.FALLING_BLOCK) {
                return p.getName() + " was squished to death";
            }
            if (cause == EntityDamageEvent.DamageCause.FIRE) {
                return p.getName() + " went up in flames";
            }
            if (cause == EntityDamageEvent.DamageCause.FIRE_TICK) {
                return p.getName() + " burned to death";
            }
            if (cause == EntityDamageEvent.DamageCause.FLY_INTO_WALL) {
                return p.getName() + " experienced kinetic energy";
            }
            if (cause == EntityDamageEvent.DamageCause.FREEZE) {
                return p.getName() + " froze to death";
            }
            if (cause == EntityDamageEvent.DamageCause.HOT_FLOOR) {
                return p.getName() + " discovered the floor was lava";
            }
            if (cause == EntityDamageEvent.DamageCause.LAVA) {
                return p.getName() + " tried to swim in lava";
            }
            if (cause == EntityDamageEvent.DamageCause.LIGHTNING) {
                return p.getName() + " was struck by lightning";
            }
            if (cause == EntityDamageEvent.DamageCause.MAGIC) {
                return p.getName() + " was killed by magic";
            }
            if (cause == EntityDamageEvent.DamageCause.POISON) {
                return p.getName() + " died from poison";
            }
            if (cause == EntityDamageEvent.DamageCause.STARVATION) {
                return p.getName() + " starved to death";
            }
            if (cause == EntityDamageEvent.DamageCause.SUFFOCATION) {
                return p.getName() + " suffocated in a wall";
            }
            if (cause == EntityDamageEvent.DamageCause.SUICIDE) {
                return p.getName() + " comitted suicide";
            }
            if (cause == EntityDamageEvent.DamageCause.THORNS) {
                return p.getName() + " was killed trying to hurt somebody else";
            }
            if (cause == EntityDamageEvent.DamageCause.VOID) {
                return p.getName() + " fell out the world";
            }
            if (cause == EntityDamageEvent.DamageCause.WITHER) {
                return p.getName() + " withered away";
            }
        }
        return p.getName() + " died";
    }
    
    @EventHandler
    public void projectileLaunchEvent(final ProjectileLaunchEvent e) {
        if (e.getEntity() instanceof Arrow) {
            final Projectile p = e.getEntity();
            if (p.getShooter() instanceof Player) {
                final Player player = (Player)p.getShooter();
                final ItemStack bow = player.getInventory().getItemInMainHand();
                if (bow.getType() == Material.BOW && !bow.getItemMeta().getDisplayName().equals("Bow")) {
                    this.bowNames.put(player, bow.getItemMeta().getDisplayName());
                }
            }
        }
    }
    
    @EventHandler
    public void playerQuitEvent(final PlayerQuitEvent e) {
        if (this.bowNames.containsKey(e.getPlayer())) {
            this.bowNames.remove(e.getPlayer());
        }
    }
    
    public List<Block> getBlocksInHitbox(final Player p) {
        final BoundingBox b = p.getBoundingBox();
        final Vector loc = p.getLocation().toVector();
        int x = -1;
        int y = -1;
        int z = -1;
        final World w = p.getWorld();
        final List<Block> blocks = new ArrayList<Block>();
        while (x != 4) {
            while (y != 4) {
                while (z != 4) {
                    final Vector v = new Vector(loc.getX() + x, loc.getY() + y, loc.getZ() + z);
                    if (b.contains(v)) {
                        final Block block = w.getBlockAt(v.getBlockX(), v.getBlockX(), v.getBlockZ());
                        if (block != null && block.getType() != Material.AIR) {
                            blocks.add(block);
                        }
                    }
                    ++z;
                }
                ++y;
            }
            ++x;
        }
        return blocks;
    }
}
